#include "Arduino.h"
#if !defined(SERIAL_PORT_MONITOR)
  #error "Arduino version not supported. Please update your IDE to the latest version."
#endif

#if defined(SERIAL_PORT_USBVIRTUAL)
  // Shield Jumper on HW (for Leonardo and Due)
  #define port SERIAL_PORT_HARDWARE
  #define pcSerial SERIAL_PORT_USBVIRTUAL
#else
  // Shield Jumper on SW (using pins 12/13 or 8/9 as RX/TX)
  #include "SoftwareSerial.h"
  SoftwareSerial port(12, 13);
  #define pcSerial SERIAL_PORT_MONITOR
#endif

#include "EasyVR.h"

EasyVR easyvr(port);

//Groups and Commands
enum Groups
{
  GROUP_0  = 0,
  GROUP_1  = 1,
};

enum Group0 
{
  G0_READY = 0,
};

enum Group1 
{
  G1_GO = 0,
  G1_STOP = 1,
  G1_SPEED_UP = 2,
  G1_SLOW_DOWN = 3,
  G1_TURN_LEFT = 4,
  G1_TURN_RIGHT = 5,
  G1_TURN_OFF = 6,
};


int8_t group, idx;

void setup()
{
  // setup PC serial port
  pinMode(9,OUTPUT);
  pinMode(11,OUTPUT);
  pcSerial.begin(9600);

  // bridge mode?
  int mode = easyvr.bridgeRequested(pcSerial);
  switch (mode)
  {
  case EasyVR::BRIDGE_NONE:
    // setup EasyVR serial port
    port.begin(9600);
    // run normally
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge not started!"));
    break;
    
  case EasyVR::BRIDGE_NORMAL:
    // setup EasyVR serial port (low speed)
    port.begin(9600);
    // soft-connect the two serial ports (PC and EasyVR)
    easyvr.bridgeLoop(pcSerial);
    // resume normally if aborted
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge connection aborted!"));
    break;
    
  case EasyVR::BRIDGE_BOOT:
    // setup EasyVR serial port (high speed)
    port.begin(115200);
    // soft-connect the two serial ports (PC and EasyVR)
    easyvr.bridgeLoop(pcSerial);
    // resume normally if aborted
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge connection aborted!"));
    break;
  }

  while (!easyvr.detect())
  {
    Serial.println("EasyVR not detected!");
    delay(1000);
  }

  easyvr.setPinOutput(EasyVR::IO1, LOW);
  Serial.println("EasyVR detected!");
  easyvr.setTimeout(5);
  easyvr.setLanguage(0);

  group = EasyVR::TRIGGER; //<-- start group (customize)
}

void action();

void loop()
{
  if (easyvr.getID() < EasyVR::EASYVR3)
    easyvr.setPinOutput(EasyVR::IO1, HIGH); // LED on (listening)

  Serial.print("Say a command in Group ");
  Serial.println(group);
  easyvr.recognizeCommand(group);

  do
  {
    // can do some processing while waiting for a spoken command
  }
  while (!easyvr.hasFinished());
  
  if (easyvr.getID() < EasyVR::EASYVR3)
    easyvr.setPinOutput(EasyVR::IO1, LOW); // LED off

  idx = easyvr.getWord();
  if (idx >= 0)
  {
    // built-in trigger (ROBOT)
    group = GROUP_1; //<-- jump to another group X
    return;
  }
  idx = easyvr.getCommand();
  if (idx >= 0)
  {
    // print debug message
    uint8_t train = 0;
    char name[32];
    Serial.print("Command: ");
    Serial.print(idx);
    if (easyvr.dumpCommand(group, idx, name, train))
    {
      Serial.print(" = ");
      Serial.println(name);
    }
    else
      Serial.println();
	// beep
    easyvr.playSound(0, EasyVR::VOL_FULL);
    // perform some action
    action();
  }
  else // errors or timeout
  {
    if (easyvr.isTimeout())
      Serial.println("Timed out, try again...");
    int16_t err = easyvr.getError();
    if (err >= 0)
    {
      Serial.print("Error ");
      Serial.println(err, HEX);
    }
  }
}

void action()
{
    switch (group)
    {
    case GROUP_0:
      switch (idx)
      {
      case G0_READY:
        // write your action code here
        group = GROUP_1; //<-- or jump to another group X for composite commands
        break;
      }
      break;
    case GROUP_1:
      switch (idx)
      {
      case G1_GO:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
         analogWrite(9,150);
         analogWrite(11,150);
        break;
      case G1_STOP:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
        analogWrite(9,0);
        analogWrite(11,0);
        break;
      case G1_SPEED_UP:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
        analogWrite(9,225);
        analogWrite(11,225);
        break;
      case G1_SLOW_DOWN:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
        analogWrite(9,110);
        analogWrite(11,110);
        break;
      case G1_TURN_LEFT:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
        unsigned long timeStart;
        timeStart = millis();
        unsigned long timeEnd;
        timeEnd = timeStart;
        while((timeEnd - timeStart) <= 3000){
          analogWrite(9,0);
          analogWrite(11,225);
          timeEnd = millis();
        }
        break;
      case G1_TURN_RIGHT:
        // write your action code here
        // group = GROUP_X; <-- or jump to another group X for composite commands
        unsigned long timeStart1;
        timeStart1 = millis();
        unsigned long timeEnd1;
        timeEnd1 = timeStart1;
        while((timeEnd1 - timeStart1) <= 3000){
          analogWrite(9,225);
          analogWrite(11,0);
          timeEnd1 = millis();
        }
        break;
      case G1_TURN_OFF:
        // write your action code here
        analogWrite(9,0);
        analogWrite(11,0);
        group = GROUP_0; //<-- or jump to another group X for composite commands
        break;
      }
      break;
    }
}
